
use field_offset::offset_of;

pub struct Vertex {
    Position: glm::Vec3,
    Normal: glm::Vec2,
    TexCoords: glm::Vec2,
    Tangent: glm::Vec3,
    Bitangent: glm::Vec3,
    m_BoneIDs: [i32; 4],
    m_Weights: [f32; 4],
}

pub struct Texture {
    id: u32,
    Type: String,
    path: String,
}

struct Mesh {
    pub vertices: Vec<Vertex>,
    pub indices: Vec<u32>,
    pub textures: Vec<Texture>,
    pub VAO: Option<u32>,
    VBO: Option<u32>,
    EBO: Option<u32>,
}





impl Mesh{
    pub fn new(vertices: Vec<Vertex>, indices: Vec<u32>, textures: Vec<Texture>)
        {
            let mut temp = Mesh {
                vertices,
                indices,
                textures,
                VAO: None,
                VBO: None,
                EBO: None,

            };

        }
     fn setupMesh(mut temp: Mesh)
                {
                    // create buffers/arrays

                    unsafe {

                        gl::GenVertexArrays(1, temp.VAO.as_mut().unwrap());
                        gl::GenBuffers(1, temp.VBO.as_mut().unwrap());
                        gl::GenBuffers(1, temp.EBO.as_mut().unwrap());

                        gl::BindVertexArray(temp.VAO.unwrap());


                        // NOTE, i am not sure if this one will work
                        gl::BindBuffer(gl::ARRAY_BUFFER, temp.VBO.unwrap());
                        gl::BufferData(gl::ARRAY_BUFFER, std::mem::size_of_val(&temp.vertices) as isize, temp.vertices.as_ptr().cast(), gl::STATIC_DRAW);

                        // the vertex attribPointers
                        // vert pox
                        gl::EnableVertexAttribArray(0);
                        gl::VertexAttribPointer(0, 3, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, 0 as *const _);
                        // vertex normals
                        gl::EnableVertexAttribArray(1);
                        gl::VertexAttribPointer(1, 3, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => Normal).get_byte_offset() as *const _);

                        // vertex texture coords

                        gl::EnableVertexAttribArray(2);
                        gl::VertexAttribPointer(2, 2, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => TexCoords).get_byte_offset() as *const _);

                        //vertex tangent

                        gl::EnableVertexAttribArray(3);
                        gl::VertexAttribPointer(3, 3, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => Tangent).get_byte_offset() as *const _);

                        // vertex Bitangent

                        gl::EnableVertexAttribArray(4);
                        gl::VertexAttribPointer(4, 3, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => Bitangent).get_byte_offset() as *const _);

                        // ids
                        gl::EnableVertexAttribArray(5);
                        gl::VertexAttribIPointer(5, 4, gl::INT, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => m_BoneIDs).get_byte_offset() as *const _);

                        // weights
                        gl::EnableVertexAttribArray(6);
                        gl::VertexAttribPointer(6, 4, gl::FLOAT, gl::FALSE, std::mem::size_of::<Vertex>() as i32, offset_of!(Vertex => m_Weights).get_byte_offset() as *const _);
                    }

                }

}
